"""\
$id$

This file is an installation script for this skin.  
It is meant to be used as an External Method.  

To use, either use Plone 2.0+ to install, or add an external 
method to the root of the Plone Site that you want the skin registered 
in with the configuration:

 id: install_skin
 title: Install Skin *optional*
 module name: GoogleSATool.Install
 function name: install

Then go to the management screen for the newly added external method
and click the 'Try it' tab.  
"""
import sys
from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import ManagePortal

from cStringIO import StringIO
import string

from Products.GoogleSATool import googlesa_globals
from Products.GoogleSATool.config import CONFIGLETNAME, PROJECTNAME, TOOLTITLE, TOOLNAME
from zExceptions import BadRequest

skin_names = ('GoogleSATool',)

def install_tool(self, out):
    """Install the Google Search Appliance tool
    """
    # add the library tool
    addTool = self.manage_addProduct[PROJECTNAME].manage_addTool
    try:
        addTool(PROJECTNAME)
        print >>out, "Added the %s Tool to the plone Site" % TOOLTITLE
        return
    except BadRequest:
        pass
    except: # Older Zopes
        #heuristics for testing if an instance with the same name already exists
        #only this error will be swallowed.
        #Zope raises in an unelegant manner a 'Bad Request' error
        e=sys.exc_info()
        if e[0] != 'Bad Request':
            raise
    print >>out, "%s Tool already added" % TOOLTITLE

def uninstall_tool(self, out):
    try:
        self.manage_delObjects([TOOLNAME])
    except:
        pass
    else:
        print >>out, "%s tool removed" % TOOLTITLE

def install_configlet(self, out):
    try:
        portal_conf=getToolByName(self,'portal_controlpanel')
    except AttributeError:
        print >>out, "Configlet could not be installed"
        return
    try:
        portal_conf.registerConfiglet(CONFIGLETNAME
               , TOOLTITLE
               , 'string:${portal_url}/%s/google_settings' % TOOLNAME
               , ''                 # a condition   
               , 'Manage portal'    # access permission
               , 'Products'         # section to which the configlet should be added: 
                                    #(Plone,Products,Members) 
               , 1                  # visibility
               , PROJECTNAME
               , 'googlesa_icon.ico' # icon in control_panel
               , TOOLTITLE
               , None
               )
    except KeyError:
        pass # Get KeyError when registering duplicate configlet.

def uninstall_configlet(self, out):
    # remove the configlet from the portal control panel
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        configTool.unregisterConfiglet(CONFIGLETNAME)
        out.write('Removed %s configlet\n' % TOOLTITLE)

def setupSkin(self, out):
    skinsTool = getToolByName(self, 'portal_skins')
    
    # Add directory views
    try:  
        addDirectoryViews(skinsTool, 'skins', googlesa_globals)
        out.write( "Added directory views to portal_skins.\n" )
    except:
        out.write( '*** Unable to add directory views to portal_skins.\n')

    def firstplone(path):
        # Return the index of the first plone skin layer
        # ecmascript usually comes first, but plone_forms is the one
        # we really need to come before.
        ploneskins = ['plone_ecmascript', 'plone_forms']
        return min([path.index(layer) for layer in ploneskins])
        
    # Go through the skin configurations and insert the skin
    skins = skinsTool.getSkinSelections()
    for skin in skins:
        path = skinsTool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        changed = 0
        for skin_name in skin_names:
            if skin_name not in path:
                try: 
                    path.insert(firstplone(path), skin_name)
                    changed = 1
                except ValueError:
                    path.append(skin_name)
                    changed = 1

        if changed:        
            path = string.join(path, ', ')
            # addSkinSelection will replace existing skins as well.
            skinsTool.addSkinSelection(skin, path)
            out.write("Added %s to %s skin\n" % (', '.join(skin_names),skin))
        else:
            out.write("Skipping %s skin, %s already set up\n" % (skin, ', '.join(skin_names)))

def install_customisation(self, out):
    """Default settings may be stored in a customisation policy script so
    that the entire setup may be 'productised'"""

    # Skins are cached during the request so (in case new skin
    # folders have just been added) we need to force a refresh of the
    # skin.
    self.changeSkin(None)

    scriptname = '%s-customisation-policy' % PROJECTNAME.lower()
    cpscript = getattr(self, scriptname, None)

    if not cpscript:
        # Try default skin
        getDefaultSkin = getattr(self, 'getDefaultSkin', None)
        if getDefaultSkin:
            self.changeSkin(getDefaultSkin())
        cpscript = getattr(self, scriptname, None)
        self.changeSkin(None)

    # If the user hasn't created a CP then use the sample.
    if not cpscript:
        cpscript = getattr(self, 'sample-'+scriptname, None)
        
    if cpscript:
        cpscript = cpscript.__of__(self)

    if cpscript:
        print >>out,"Customising %s" % PROJECTNAME
        print >>out,cpscript()
    else:
        print >>out,"No customisation policy"

def install(self):
    out=StringIO()

    install_tool(self, out)
    install_configlet(self, out)
    setupSkin(self, out)
    install_customisation(self, out)
    out.write('Installation completed.\n')
    return out.getvalue()

def unregister_layers(self, names, out):
    """Remove a directory from the skins"""
    skinstool = getToolByName(self, 'portal_skins')

    # remove this layer from all known skins
    for skinName in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skinName) 
        path = [i.strip() for i in path.split(',')]

        opath = list(path)
        for name in names:
            if name in path:
                path.remove(name)

        if opath != path:
            path = ','.join(path)
            skinstool.addSkinSelection(skinName, path)

            print >>out, "removed layers '%s' from '%s'" % (', '.join(names), skinName)
    self.changeSkin(None)

def uninstall(self):
    out = StringIO()

    unregister_layers(self, skin_names, out)
    uninstall_configlet(self, out)
    uninstall_tool(self, out)

    print >> out, "Successfully uninstalled %s." % PROJECTNAME
    return out.getvalue()
