# Code to manipulate GoogleSA search strings.
import re

def FieldDef(name, default=None, required=False, ):
    deletable = name not in ('client', 'site', 'q', 'output', '')
    if name in ('client', 'site', 'output'):
        required = True

    if not default:
        default = None

    return dict(name=name,
        default=default,
        required=required,
        deletable=deletable,
        listsep=SEPARATORS.get(name, None),
        assumed = ASSUMED.get(name, ''))

# Some fields can have lists, and they don't all use the same
# separator.
SEPARATORS = {
    'site': '|',
    'requiredfields': '.',
    'partialfields': '.',
    'getfields': '.',
}

ASSUMED = {
    'num': 10,
    'numgm': 3,
    'filter': 1,
    'ie': 'latin1',
    'oe': 'latin1',
    'proxyreload':0,
    'start':0,
    'access': 'p',
    'sort': 'date:D:L:d1',
    'as_dt': 'i',
    }

DEFAULT_FIELDS = dict([(f['name'],f) for f in [
    FieldDef('access'),
    FieldDef('as_dt', 'request/as_dt'),
    FieldDef('as_sitesearch', 'request/as_sitesearch'),
    FieldDef('client', 'string:ENTER A CLIENT NAME'),
    FieldDef('filter', 'request/filter'),
    FieldDef('lr', 'request/language'),
    FieldDef('num', 'request/size'),
    FieldDef('numgm'),
    FieldDef('oe', 'string:utf-8'),
    FieldDef('output', 'string:xml'),
    FieldDef('partialfields', 'request/partialfields'),
    FieldDef('q', 'request/SearchableText'),
    FieldDef('requiredfields', 'request/requiredfields'),
    FieldDef('site', 'string:ENTER A SITE NAME'),
    FieldDef('sort', 'request/sort'),
    FieldDef('start', 'request/start'),
    ]])

def listValidator(*values):
    def validator(self, value):
        if not value:
            return None
        return value in values

class FieldValidation:
    validate_access = listValidator('p','s','a')
    validate_as_dt = listValidator('i','e')
    validate_as_occt = listValidator('any', 'title', 'URL')
    validate_filter = listValidator(0, 1)

    def validate_num(self, value):
        if isinstance(value, basestring):
            value = int(value)
        return 2 <= value <= 100

    def validate_site(self, value):
        regex = getattr(self, '_collectionRegex', '')
        if regex:
            for v in value:
                match = re.match(regex, v)
                if match is None or match.group(0) != v:
                    return False
        return True
