##############################################################################
#
# Copyright (c) 2005 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" GoogleSA tool setup handlers.
"""
import os

from AccessControl import ClassSecurityInfo
from AccessControl.Permission import Permission
from Globals import InitializeClass
from Globals import package_home
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.CMFCore.utils import getToolByName

from Products.GenericSetup.utils import CONVERTER
from Products.GenericSetup.utils import DEFAULT
from Products.GenericSetup.utils import ExportConfiguratorBase
from Products.GenericSetup.utils import ImportConfiguratorBase
from Products.GenericSetup.utils import KEY

from Products.CMFCore.permissions import ManagePortal
from Products.PythonScripts.standard import Object

_pkgdir = package_home( globals() )
_xmldir = os.path.join( _pkgdir, 'xml' )

#
#   Configurator entry points
#
_FILENAME = 'googlesa.xml'

def importGoogleSASettings(context):
    """ Import gsa settings from an XML file.
    """
    site = context.getSite()
    gt = getToolByName(site, 'portal_googlesa', None)
    if gt is None:
        return 'GoogleSA: No tool!'

    body = context.readDataFile(_FILENAME)
    if body is None:
        return 'GoogleSA: Nothing to import.'

#     if context.shouldPurge():
#         ait.clearActionIcons()

    # now act on the settings we've retrieved
    configurator = GSAToolImportConfigurator(site, 'utf-8')
    gt_info = configurator.parseXML(body)

    url = gt_info['url']
    collectionRegex = gt_info['collections']
    fields = [ Object(**f) for f in gt_info['fields'] ]
    searches = [ Object(**s) for s in gt_info['searches']]

    gt.manage_setGooglePrefs(
        googleURL=url,
        fields=fields,
        collectionRegex=collectionRegex,
        searches=searches)
    
    return 'GSA settings imported.'

def exportGoogleSASettings(context):
    """ Export caching policy manager settings as an XML file.
    """
    site = context.getSite()
    mhc = GSAToolExportConfigurator( site ).__of__( site )
    text = mhc.generateXML()

    context.writeDataFile( _FILENAME, text, 'text/xml' )

    return 'GSA tool settings exported.'


class GSAToolExportConfigurator(ExportConfiguratorBase):
    """ Synthesize XML description of cc properties.
    """
    security = ClassSecurityInfo()

    security.declareProtected(ManagePortal, 'getTool')
    def getTool(self):
        """Get the GoogleSA tool"""
        gt = getToolByName(self._site, 'portal_googlesa')
        return gt

    def listFieldInfo(self):
        """ Return a list of mappings describing the Google fields.
        """
        gt = self.getTool()
        for info in gt.getFieldOptions().itervalues():
            yield info

    def listSearchInfo(self):
        """ Return a list of mappings describing the canned searches.
        """
        gt = self.getTool()
        for info in gt.getEditExtraSearches()[:-1]:
            yield info
        
    security.declarePrivate('_getExportTemplate')
    def _getExportTemplate(self):
        return PageTemplateFile('gsaExport.xml', _xmldir)

InitializeClass(GSAToolExportConfigurator)


class GSAToolImportConfigurator(ImportConfiguratorBase):

    def _getImportMapping(self):
        return {
            'google-settings':
                {
                    'url': {},
                    'collections': {},
                    'field': {KEY:'fields' },
                    'search': { KEY:'searches' }
                },
            'field':
                {
                    'name': {},
                    'default': { DEFAULT: ""},
                    'required': {CONVERTER: self._convertToBoolean, DEFAULT: False},
                },
            'search':
                {
                    'title': { DEFAULT: ""},
                    'query': { DEFAULT: ""},
                },
    }

InitializeClass(GSAToolImportConfigurator)
