import pydoc
import traceback
import inspect

__all__ = ['MyHTMLDoc']
class MyHTMLDoc(pydoc.HTMLDoc):
    """Override HTMLDoc so we can insert the output in a page template"""
    def page(self, title, contents):
        """Format an HTML page"""
        return contents

    def classlink(self, object, modname):
        """Pydoc version fails on extension classes."""
        try:
            return pydoc.HTMLDoc.classlink(self, object, modname)
        except AttributeError:
            return object.__name__

    def docclass(self, object, *args, **kw):
        try:
            return pydoc.HTMLDoc.docclass(self, object, *args, **kw)
        except AttributeError:
            return "Sorry, cannot document %s\n%s" % (object.__name__, traceback.format_exc())
            
def fixed_dir(object):
    """Hotfix for inspect so it doesn't die on Zope extension classes"""
    if hasattr(object, '__dict__'):
        res = [name
            for name in dir(object)
            if name in object.__dict__ or getattr(object, name, None) is not None]
    else:
        res = [name
            for name in dir(object)
            if getattr(object, name, None) is not None]

    if inspect.ismodule(object) and hasattr(object, '__all__'):
        all = object.__all__
        res = [name for name in res if name in all]

    res = [name for name in res if not name.endswith('__roles__') ]
    return res
    
inspect.dir = fixed_dir
