﻿# -*- coding: UTF-8 -*-
################################################################################
#
# Copyright (c) 2002-2005, Oxfam. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the author nor the names of its contributors may be used
#   to endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
################################################################################
from Testing import ZopeTestCase
from Testing.ZopeTestCase.functional import Functional
from Products.CMFCore.utils import getToolByName
from Products.GoogleSATool.tests import z27transaction as transaction

DEPS = ('MimetypesRegistry', 'PortalTransforms', 'Archetypes', 'ATContentTypes', 'GoogleSATool')

for product in DEPS:
    ZopeTestCase.installProduct(product, 1)

from Products.CMFPlone.tests import PloneTestCase
from Products.CMFPlone.tests.PloneTestCase import portal_name, portal_owner

from AccessControl import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager, setSecurityManager
from AccessControl.SecurityManagement import noSecurityManager
from Acquisition import aq_base
import time
from StringIO import StringIO

from Products.GoogleSATool.Extensions import Install as installGSA
try:
    from Products.ATContentTypes.lib import constraintypes
except:
    constraintypes = None

default_user = ZopeTestCase.user_name

# Type names vary according to the version of Plone and/or
# ATContentTypes
TypeMapping = {
    'ATDocument': 'ATDocument',
    'ATImage': 'ATImage',
    'ATLink': 'ATLink',
    'ATFolder': 'ATFolder',
}
def MapType(typename):
    return TypeMapping[typename]

DESTINATION_FOLDERS = {'ATDocument':'documents',
                       'ATImage':'images',
                       'ATLink':'links',
                       'ATFolder':'folders',
                      }

def setupGSA(id=portal_name, quiet=0):

    # Set up CMS site.
    app = ZopeTestCase.app()

    portal = app[id]
    quickinstaller = portal.portal_quickinstaller
    quickinstaller.installProduct('ATContentTypes')

    # Fix up types so the tests work both on Plone 2.0 and 2.1
    typestool = portal.portal_types
    if not hasattr(typestool, 'ATDocument'):
        # Use the type names without the AT prefix
        TypeMapping.update({
            'ATDocument': 'Document',
            'ATImage': 'Image',
            'ATLink': 'Link',
            'ATFolder': 'Folder',
            })

    _start = time.time()
    if not quiet: ZopeTestCase._print('Adding GSA tool ... ')
    # Login as portal owner
    user = app.acl_users.getUserById(portal_owner).__of__(app.acl_users)
    newSecurityManager(None, user)
    installGSA.install(portal)

    # Log out
    noSecurityManager()
    transaction.commit()
    if not quiet: ZopeTestCase._print('done (%.3fs)\n' \
                                      % (time.time()-_start,))

    ZopeTestCase.close(app)
        
class GSATestCase(PloneTestCase.PloneTestCase):
    
    def afterSetUp(self):
        self.committed = 0

        # we need to have the Manager role to be able to add things
        # to the portal root
        self.setRoles(['Manager',])
        sm = getSecurityManager()
        try:
            self.loginPortalOwner()

            portal = self.portal

            # Add a manager user
            uf = self.portal.acl_users
            uf._doAddUser('manager', 'secret', ['Manager'], [])
            # Set ordinary user roles
            self.setRoles(['Member'], default_user)
        finally:
            setSecurityManager(sm)
        self.loginDefaultUser()

    def loginDefaultUser(self):
        '''Use if you need to manipulate the portal itself.'''
        newSecurityManager(None, self.getMemberUser())

    def loginManagerUser(self):
        '''Use if you need to manipulate the portal itself.'''
        newSecurityManager(None, self.getManagerUser())

    def loginPortalOwner(self):
        '''Use if you need to manipulate the portal itself.'''
        uf = self.app.acl_users
        user = uf.getUserById(portal_owner).__of__(uf)
        newSecurityManager(None, user)

    def getManagerUser(self):
        # b/w compat
        uf = self.portal.acl_users
        return uf.getUserById('manager').__of__(uf)

    def getMemberUser(self):
        # b/w compat
        uf = self.portal.acl_users
        return uf.getUserById(default_user).__of__(uf)

class GSAFunctionalTestCase(Functional, GSATestCase):
    """test case for functional tests inside a Plone with GSA tool
    """
    __implements__ = Functional.__implements__ + GSATestCase.__implements__
    
setupGSA()

