##############################################################################
#
# Copyright (c) 2003-2005 Oxfam. All rights reserved.
#
# This software is distributed under the terms of the GNU Lesser
# General Public License (LGPL) License. See LICENSE.txt for license
# text.
#
##############################################################################
"""Tests for the XML to Python class hierarchy

$Id$
"""

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

import Acquisition
from Testing.ZopeTestCase import ZopeTestCase
# from Products.CMFPlone.tests import PloneTestCase
import Products.GoogleSATool
from Products.GoogleSATool import GoogleResults, googlesa_globals
from Globals import package_home
from DateTime import DateTime

def loadfile(name):
    fullname = os.path.join(package_home(googlesa_globals), name)
    f = open(fullname, 'r')
    data = f.readlines()
    f.close()
    return data

class TestParsing(ZopeTestCase):
    """Test the implementation of the result parsing"""

    def afterSetUp(self):
        #self.portal = FakePortal()
        pass

    def parseFile(self, name):
        fullname = os.path.join(package_home(googlesa_globals), name)
        try:
            f = open(fullname, 'r')
            return GoogleResults.parse(f)
        finally:
            f.close()

    def test_parse1(self):
        results = self.parseFile('tests/input/water.xml')
        self.assertEquals(results.Q, 'water')
        self.failUnless(results.TM - 0.115258 < 0.0000001)
        RES = results.RES
        self.assertEquals(RES.M, 2780)
        R = RES.R
        self.assertEquals(len(R), 10)
        self.assertEquals(R[3].N, 4)
        self.assertEquals(R[3].L, 1)
        self.assertEquals(R[7].L, 2)

    def test_paramdict(self):
        results = self.parseFile('tests/input/water.xml')
        self.assertEquals(results.PARAM["site"].value, "sss")
        self.assertEquals(results.PARAM["output"].name, "output")

        FS = results.RES.R[0].FS
        self.assertEquals(FS['date'].VALUE, '2005-07-22')
        FS = results.RES.R[1].FS
        self.assertEquals(FS['date'].VALUE, '')

    def test_filtering(self):
        results = self.parseFile('tests/input/water.xml')
        self.assertEquals(results.RES.FI, True)

if __name__ == '__main__':
    framework()
else:
    # While framework.py provides its own test_suite()
    # method the testrunner utility does not.
    from unittest import TestSuite, makeSuite
    def test_suite():
        suite = TestSuite()
        suite.addTest(makeSuite(TestParsing))
        return suite
