# Backward compatability functions.
import copy

try: reversed ## Introduced in 2.4
except NameError:
    def reversed(seq):
        """Iterate over x in reverse order.
        >>> list(reversed([1,2,3]))
        [3, 2, 1]
        """
        if hasattr(seq, 'keys'):
            raise ValueError("mappings do not support reverse iteration")
        i = len(seq)
        while i > 0:
            i -= 1
            yield seq[i]


try: sorted ## Introduced in 2.4
except NameError:
    def sorted(seq, cmp=None, key=None, reverse=False):
        """Partial implementation of the "sorted" function from Python 2.4"""
        if cmp is not None:
            raise NotImplementedError('cmp')

        if key is None:
            def key(v):
                return v

        sign = reverse and -1 or 1
        lst = [(key(i), sign*j, i) for j,i in enumerate(seq)] 
        lst.sort() 
        if reverse: 
            lst = reversed(lst) 
        return [i for __, __, i in lst]
